'use strict'

const opn = require('opn')
const chalk = require('chalk')
const logger = require('../util/logger')
const compilerHandler = require('../webpack/handler')

const taskName = 'dev'
const env = require('../lib/config').ENV_DEVELOPMENT
const serverOptions = {
  port: 8080,
  open: false, // 是否自动打开浏览器
  historyFallback: true, // 开启后404返回index.html
  publicPath: ''
}

module.exports = function (workflow, options) {
  const gulp = workflow.gulp

  if (workflow.task === taskName) {
    workflow.setEnv(env)
    init(workflow)
  }

  gulp.task(taskName, done => {
    const server = workflow.server
    const webpackConfig = workflow.compilerOptions

    if (server) {
      prependHotClient(webpackConfig)
    }

    const compiler = workflow.initCompiler(webpackConfig)

    if (server) {
      workflow.emit('beforeServerStart', server)
      startServer(server, compiler)
      workflow.emit('serverStart', server)
    } else {
      logger.info('webpack watching...')
      compiler.watch({}, compilerHandler)
    }
  })
}

function init (workflow) {
  const config = workflow.config
  const webpackConfig = require('../webpack/config.dev')(config, env)
  const userServerOptions = config.server

  if (userServerOptions) {
    workflow.initServer()
    if (typeof userServerOptions === 'object') {
      Object.assign(serverOptions, userServerOptions)
    }
  }

  webpackConfig.output.publicPath = serverOptions.publicPath
  workflow.setCompilerOptions(webpackConfig)
}

function startServer (server, compiler) {
  const port = serverOptions.port
  const publicPath = serverOptions.publicPath
  const startUrl = 'http://localhost:' + port + publicPath + '/index.html'
  const devMiddleware = getDevMiddleware(compiler)
  const hotMiddleware = getHotMiddleware(compiler)

  devMiddleware.waitUntilValid(function () {
    server.listen(port, () => {
      logger.success('server listening at %s', chalk.underline(startUrl))
    })
    if (serverOptions.open) {
      opn(startUrl)
    }
  })
  compiler.plugin('done', function () {
    logger.success('webpack compile success')
  })
  // html刷新
  compiler.plugin('compilation', compilation => {
    compilation.plugin('html-webpack-plugin-after-emit', (data, cb) => {
      hotMiddleware.publish({ action: 'reload' })
      cb()
    })
  })

  serverOptions.historyFallback && server.use(require('connect-history-api-fallback')())
  server.use(devMiddleware)
  server.use(hotMiddleware)
}

function getDevMiddleware (compiler) {
  const webpackDevMiddleware = require('webpack-dev-middleware')
  return webpackDevMiddleware(compiler, {
    publicPath: compiler.options.output.publicPath,
    noInfo: true,
    lazy: false,
    stats: {
      colors: true
    }
  })
}

function getHotMiddleware (compiler) {
  const webpackHotMiddleware = require('webpack-hot-middleware')
  return webpackHotMiddleware(compiler, {
    log: function () {}
  })
}

function prependHotClient (webpackConfig) {
  const entry = webpackConfig.entry
  const hotClient = require.resolve('../webpack/hot-client')

  if (!entry) return
  if (Array.isArray(entry) || typeof entry === 'string') {
    webpackConfig.entry = [hotClient].concat(entry)
  } else {
    Object.keys(entry).forEach(name => {
      entry[name] = [hotClient].concat(entry[name])
    })
  }
}
