'use strict'

const path = require('path')
const webpack = require('webpack')
const merge = require('webpack-merge')
const profile = require('../lib/setup').getProfile()
const modulePaths = [
  'node_modules',
  profile.basedir + '/node_modules',
  path.resolve(__dirname, '../node_modules'),
  path.join(__dirname, '../../') // 在依赖被打平的情况下，在项目存放的同级目录里查找
]
const parseConfig = require('./parse-config')
const hashFilename = require('./hash-filename')
const ENV_PRODUCTION = require('../lib/config').ENV_PRODUCTION
const isWindows = process.platform === 'win32'

module.exports = function (config, env) {
  const userOptions = parseConfig(config, env)

  function getFilename (filename) {
    if (env === ENV_PRODUCTION && config.hash) {
      return hashFilename(filename, 'hash')
    }
    return filename
  }

  function genTranspileDepRegex (transpileDependencies) {
    const deps = transpileDependencies.map(dep => {
      if (typeof dep === 'string') {
        const depPath = path.join('node_modules', dep, '/')
        return isWindows
          ? depPath.replace(/\\/g, '\\\\') // double escape for windows style path
          : depPath
      } else if (dep instanceof RegExp) {
        return dep.source
      }
    })
    return deps.length ? new RegExp(deps.join('|')) : null
  }

  const minifyImages = config.minifyImages
  const imgReg = /\.(png|jpe?g|gif|webp|svg)(\?.*)?$/
  return merge({
    module: {
      rules: [
        {
          test: /\.js$/,
          loader: 'babel-loader',
          exclude: function (filepath) {
            const transpileDepRegex = genTranspileDepRegex(config.transpileDependencies)
            if (transpileDepRegex && transpileDepRegex.test(filepath)) {
              return false
            }
            return /node_modules/.test(filepath)
          },
          options: {
            presets: [
              require('babel-preset-env')
            ],
            plugins: [
              require('babel-plugin-transform-runtime'),
              require('babel-plugin-transform-object-rest-spread'),
            ],
            ...config.babel
          }
        },
        {
          test: imgReg,
          loaders: [
            {
              loader: 'url-loader',
              options: {
                name: getFilename('img/[name].[ext]'),
                limit: 1000
              }
            }
          ]
        },
        ...minifyImages ? [{
          test: imgReg,
          loaders: [
            {
              loader: 'image-webpack-loader',
              options: minifyImages.loaderRawOptions
                ? minifyImages.loaderRawOptions
                : {
                mozjpeg: {
                  progressive: minifyImages.jpgProgressive,
                  quality: minifyImages.quality
                },
                // optipng.enabled: false will disable optipng
                optipng: {
                  enabled: minifyImages.optipngEnabled
                },
                pngquant: {
                  quality: minifyImages.pngQuality,
                  speed: minifyImages.pngSpeed
                },
                gifsicle: {
                  interlaced: minifyImages.gifInterlaced
                }
              }
            }
          ],
          exclude: minifyImages.exclude
        }]
        : [],
        {
          test: /\.(mp4|m3u8|mp3)(\?.*)?$/,
          loader: 'file-loader',
          options: {
            name: getFilename('[name].[ext]')
          }
        },
        {
          test: /\.(woff2?|eot|ttf|otf|svg)(\?.*)?$/,
          loader: 'url-loader',
          options: {
            name: getFilename('font/[name].[ext]'),
            limit: 1000
          }
        },
        {
          test: /\.html$/,
          loader: 'html-loader',
          options: {
            attributes: true,
            minimize: {
              collapseWhitespace: true,
              conservativeCollapse: true,
              keepClosingSlash: true,
              minifyCSS: true,
              minifyJS: true,
              removeAttributeQuotes: false,
              removeComments: true,
              removeScriptTypeAttributes: true,
              removeStyleLinkTypeAttributes: true,
              useShortDoctype: true
            }
          }
        }
      ]
    },
    plugins: [
      new webpack.DefinePlugin({
        'process.env.NODE_ENV': JSON.stringify(env)
      })
    ],
    resolve: {
      modules: modulePaths
    },
    resolveLoader: {
      modules: modulePaths
    }
  }, userOptions)
}
