'use strict'

const path = require('path')
const webpack = require('webpack')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const ExtractTextPlugin = require('extract-text-webpack-plugin')
const hashFilename = require('./hash-filename')
const ENV_PRODUCTION = require('../lib/config').ENV_PRODUCTION

module.exports = function (config, env) {
  config = Object.assign({
    minimize: true
  }, config)

  let output = typeof config.output === 'string'
    ? { path: config.output }
    : config.output
  const webpackConfig = {
    entry: config.entry,
    module: {
      rules: []
    },
    output,
    plugins: [].concat(initHtmlPlugins(config.template))
  }

  if (output && output.path) {
    output.path = path.resolve(process.cwd(), output.path)
  }
  if (env === ENV_PRODUCTION) {
    webpackConfig.plugins = webpackConfig.plugins.concat(
      initUglifyPlugin(config.minimize),
      initCommonsChunkPlugins(config.commonChunk)
    )
  }
  initStylesConfig(webpackConfig, config, env)

  return webpackConfig
}

function initUglifyPlugin (value) {
  if (!value) return []
  return new webpack.optimize.UglifyJsPlugin({
    compress: {
      warnings: false,
      properties: false
    },
    output: {
      keep_quoted_props: true
    },
    mangle: {
      screw_ie8: false,
      except: ['e']
    }
  })
}

function initHtmlPlugins (value) {
  let pluginOptions = []

  if (value === true) {
    pluginOptions.push({})
  } else if (typeof value === 'string') {
    pluginOptions.push({
      template: value
    })
  } else if (Array.isArray(value)) {
    pluginOptions = pluginOptions.concat(value)
  }

  return pluginOptions.map(function (options) {
    return new HtmlWebpackPlugin(options)
  })
}

function initCommonsChunkPlugins (value) {
  let pluginOptions = []

  if (value === true) {
    pluginOptions.push(
      // 第三方包或者多次引用的包
      {
        name: 'common',
        minChunks: function (module, count) {
          return (
            module.context && module.context.indexOf('node_modules') !== -1 ||
            count >= 2
          )
        }
      },
      // 第三方包
      {
        name: 'vendor',
        minChunks: function (module, count) {
          return module.context && module.context.indexOf('node_modules') !== -1
        }
      },
      // webpack runtime
      {
        name: 'manifest',
        minChunks: Infinity
      }
    )
  } else if (Array.isArray(value)) {
    pluginOptions = pluginOptions.concat(value)
  }

  return pluginOptions.map(function (options) {
    return new webpack.optimize.CommonsChunkPlugin(options)
  })
}

function initStylesConfig (webpackConfig, config) {
  const isProd = process.env.NODE_ENV === ENV_PRODUCTION
  const hash = isProd && config.hash
  const extract = isProd && config.extractCSS

  webpackConfig.module.rules.push({
    test: /\.css$/,
    use: generateCssLoaders(config)
  })

  if (extract) {
    let filename = typeof extract === 'string' ? extract : 'css/[name].css'
    if (hash) {
      filename = hashFilename(filename, 'contenthash')
    }
    webpackConfig.plugins.push(
      new ExtractTextPlugin({
        filename,
        allChunks: true
      })
    )
  }
}

function generateCssLoaders (config, options) {
  options = Object.assign({
    fallback: 'style-loader',
    loaders: []
  }, options)
  const cssLoader = {
    loader: 'css-loader',
    options: {
      minimize: config.minimize
    }
  }
  const loaders = [cssLoader].concat(options.loaders)

  if (process.env.NODE_ENV === ENV_PRODUCTION && config.extractCSS) {
    return ExtractTextPlugin.extract({
      use: loaders,
      fallback: options.fallback
    })
  } else {
    return [options.fallback].concat(loaders)
  }
}
module.exports.generateCssLoaders = generateCssLoaders
